"""Top-level package for rstms-vmwctl."""

from .cli import cli
from .version import __author__, __email__, __timestamp__, __version__

__all__ = ["cli", "__version__", "__timestamp__", "__author__", "__email__"]



"""Console script for rstms_vmwctl."""

import json
import os
import sys
from pathlib import Path

import click
import click.core
from click_params import IP_ADDRESS, MAC_ADDRESS

from .exception_handler import ExceptionHandler
from .shell import _shell_completion
from .sshconfig import SSHConfig
from .version import __timestamp__, __version__
from .vmware_workstation import Client

header = f"vmctl v{__version__} {__timestamp__}"


def _ehandler(ctx, option, debug):
    ctx.obj = dict(ehandler=ExceptionHandler(debug))
    ctx.obj["debug"] = debug


def fail(msg):
    click.echo(f"vmwctl: {msg}", err=True)
    sys.exit(-1)


def output(ctx, obj, exit=0):
    if not ctx.raw:
        obj = json.dumps(obj, indent=2)
    click.echo(obj)
    sys.exit(exit)


@click.group("vmctl", context_settings={"auto_envvar_prefix": "VMWCTL"})
@click.version_option(message=header)
@click.option("-d", "--debug", is_eager=True, is_flag=True, callback=_ehandler, help="debug mode")
@click.option("-h", "--host", envvar="VMCTL_HOST")
@click.option("-u", "--url", envvar="VMCTL_URL")
@click.option("-U", "--username", envvar="VMCTL_USERNAME")
@click.option("-P", "--password", envvar="VMCTL_PASSWORD")
@click.option("-a", "--arp-command", envvar="VMCTL_RARP", help="arp table command")
@click.option("-i", "--iso", envvar="VMCTL_ISO", help="cdrom iso pathname or URL")
@click.option("--no-iso", is_flag=True)
@click.option("-t", "--timeout", type=int, help="wait timeout in seconds")
@click.option("-r", "--raw", is_flag=True, help="suppress json formatting")
@click.option("-c", "--config-file", default=".vmctl.conf")
@click.option("-C", "--ca-file", default=None, help="certificate authority file")
@click.option("--client-cert", default=None, help="client certificate file")
@click.option("--client-key", default=None, help="client certificate key")
@click.option("-m", "--mount-path", default="/mnt", help="local mount path root")
@click.option(
    "--path-list",
    default="/vmware",
    envvar="VMCTL_PATH_LIST",
    help="comma delimited list of vmware base paths",
)
@click.option("-v", "--verbose", is_flag=True)
@click.option("-V/-A", "--vmrun/--api", is_flag=True, help="use vmware vmrun cli")
@click.option("-w/-W", "--wait/--no-wait", default=True, is_flag=True)
@click.option(
    "--shell-completion",
    is_flag=False,
    flag_value="[auto]",
    callback=_shell_completion,
    help="configure shell completion",
)
@click.pass_context
def cli(
    ctx,
    debug,
    shell_completion,
    host,
    url,
    username,
    password,
    verbose,
    path_list,
    arp_command,
    iso,
    no_iso,
    config_file,
    raw,
    vmrun,
    timeout,
    wait,
    mount_path,
    ca_file,
    client_cert,
    client_key,
):
    """vmctl top-level help"""
    config = Path(config_file)
    if not config.is_file():
        config = Path.home() / config_file
    if not config.is_file():
        config = Path.home() / ".secrets" / "vmctl.conf"
    if config.is_file():
        config = json.loads(config.read_text())
    else:
        config = {}

    host = config.get("host", host)
    url = config.get("url", url)
    username = config.get("username", username)
    password = config.get("password", password)
    path_list = config.get("path_list", path_list)
    arp_command = config.get("arp_command", arp_command)
    mount_path = config.get("mount_path", mount_path)
    ca_file = config.get("ca_file", ca_file)
    client_cert = config.get("client_cert", client_cert)
    client_key = config.get("client_key", client_key)
    iso = config.get("iso", iso)
    if no_iso:
        iso = None
    client = Client(
        host, url, username, password, path_list, arp_command, iso, mount_path, ca_file, client_cert, client_key
    )
    client.verbose = verbose
    client.raw = raw
    client.vmrun = vmrun
    if timeout:
        client.timeout = timeout
    client.wait = wait
    ctx.obj = client


@cli.command("ls")
@click.option("-l", "--long", is_flag=True)
@click.option("-v", "--vmx", is_flag=True)
@click.option("-r", "--running", is_flag=True)
@click.argument("name", required=False)
@click.pass_obj
def _list(ctx, long, vmx, running, name):
    """list VMs"""

    if vmx:
        files = ctx.get_vmx_files()
        if name:
            files = [f for f in files if f.endswith(name + ".vmx")]
        output(ctx, files)

    if name:
        vms = {name: ctx.get_vm(name)}
    else:
        vms = ctx.get_vms()

    if running:
        vms = {k: v for k, v in vms.items() if ctx.power_state(k) == "poweredOn"}

    if long:
        vms = [ctx.get_status(name) for name in vms.keys()]
    else:
        vms = list(vms.keys())

    output(ctx, vms)


@cli.command
@click.argument("name")
@click.pass_obj
def status(ctx, name):
    """VM status"""
    output(ctx, ctx.get_status(name))


@cli.command
@click.argument("name")
@click.pass_obj
def describe(ctx, name):
    """VM detail"""
    output(ctx, ctx.get_detail(name))


@cli.command
@click.argument("name")
@click.pass_obj
def state(ctx, name):
    """VM power state"""
    output(ctx, ctx.get_power(name)["power"])


@cli.command
@click.argument("name")
@click.pass_obj
def ip(ctx, name):
    """VM IP address"""
    output(ctx, ctx.get_ip(name, fail_ok=False)["ip"])


@cli.command
@click.argument("name")
@click.pass_obj
def mac(ctx, name):
    """VM MAC address"""
    output(ctx, ctx.get_mac(name, fail_ok=False)["mac"])


@cli.command
@click.argument("name")
@click.pass_obj
def edit(ctx, name):
    """edit VMX file"""
    vm = ctx.get_vm(name)
    editor = os.environ.get("EDITOR", "vi")
    os.execvp(editor, [editor, str(vm.vmx)])


@cli.command
@click.argument("name")
@click.pass_obj
def vmx(ctx, name):
    """output VMX file"""
    ctx.raw = True
    output(ctx, ctx.get_vmx(name))


@cli.command
@click.option("-i", "--iso", help="cdrom iso pathname or URL")
@click.option("--no-iso", is_flag=True)
@click.option("-c/-d", "--connect/--disconnect", is_flag=True, default=None, help="mount/dismount ISO")
@click.argument("name")
@click.pass_obj
def iso(ctx, name, iso, no_iso, connect):
    """show or modify VM CDROM ISO"""
    iso = iso or ctx.iso
    if no_iso:
        iso = None
    result = ctx.cdrom(name, iso, connect)
    output(ctx, result)


@cli.command
@click.option("-e/-d", "--enable/--disable", is_flag=True, default=None, help="disable virtual serial port")
@click.option("-p", "--pipe", help="pipe name (implies --enable)")
@click.argument("name")
@click.pass_obj
def serial(ctx, enable, pipe, name):
    """show or modify VM serial port"""
    if pipe:
        enable = True
    result = ctx.serial(name, enable, pipe)
    output(ctx, result)


@cli.command
@click.argument("name")
@click.option("-g", "--gui", is_flag=True)
@click.option("-f", "--fullscreen", is_flag=True)
@click.option("-c/-d", "--connect/--disconnect", is_flag=True, default=None, help="mount/dismount ISO")
@click.option("-i", "--iso", help="cdrom iso pathname or URL")
@click.option("--no-iso", is_flag=True)
@click.pass_obj
def start(ctx, gui, fullscreen, connect, iso, no_iso, name):
    """power on"""
    iso = iso or ctx.iso
    if no_iso:
        iso = None
    ctx.cdrom(name, iso, connect)
    _start(ctx, gui, fullscreen, name)
    result = ctx.wait_power_state(name, "start", "poweredOn", click.echo)
    output(ctx, result)


def _start(ctx, gui, fullscreen, name):
    if ctx.vmrun or gui or fullscreen:
        operation = "start"
    else:
        operation = "on"
    return ctx.set_power(name, operation, gui=gui, fullscreen=fullscreen)


@cli.command
@click.argument("name")
@click.option("-g", "--gui", is_flag=True)
@click.option("-f", "--fullscreen", is_flag=True)
@click.option("-h", "--hard", is_flag=True, help="hard shutdown")
@click.pass_obj
def reboot(ctx, hard, gui, fullscreen, name):
    """reboot"""
    if ctx.vmrun or gui or fullscreen:
        ctx.vmrun = True
        ctx.set_power(name, "reset", hard=hard)
        result = ctx.wait_power_state(name, "reset", "poweredOn", click.echo)
    else:
        operation = "off" if hard else "shutdown"
        ctx.set_power(name, operation)
        ctx.wait_power_state(name, operation, "poweredOff", click.echo)
        _start(ctx, gui, fullscreen, name)
        result = ctx.wait_power_state(name, "restart", "poweredOn", click.echo)
    output(ctx, result)


@cli.command
@click.argument("name")
@click.option("-w/-W", "--wait/--no-wait", is_flag=True, help="wait and check status")
@click.pass_obj
def poweroff(ctx, name, wait):
    """hard power down"""
    if ctx.vmrun:
        result = ctx.set_power(name, "stop", hard=True)
    else:
        result = ctx.set_power(name, "off")
    if wait is True:
        result.update(ctx.wait_power_state(name, "off", "poweredOff", click.echo))
    output(ctx, result)


@cli.command
@click.argument("name")
@click.pass_obj
def stop(ctx, name):
    """request shutdown and power off"""
    if ctx.vmrun:
        result = ctx.set_power(name, "stop", hard=False)
    else:
        result = ctx.set_power(name, "shutdown")
    result.update(ctx.wait_power_state(name, "shutdown", "poweredOff", click.echo))
    output(ctx, result)


@cli.command
@click.option("-C", "--cpu", type=int, help="cpu count")
@click.option("-R", "--ram", type=int, help="RAM in MB")
@click.option("-D", "--disk", type=int, help="disk size in MB")
@click.option("-m", "--mac", type=MAC_ADDRESS, help="ethernet MAC address")
@click.option("-i", "--iso", envvar="VMCTL_ISO", help="cdrom iso pathname or URL")
@click.option("--no-iso", is_flag=True)
@click.option("-c/-d", "--connect/--disconnect", is_flag=True, default=True, help="mount/dismount ISO")
@click.option("-g", "--gui", is_flag=True)
@click.option("-e", "--efi", is_flag=True)
@click.option("-f", "--fullscreen", is_flag=True)
@click.option("-s", "--start", is_flag=True, help="start after create")
@click.option("--time-sync/-no-time-sync", is_flag=True, default=False, help="sync time with host")
@click.option("--guest-timezone", default="UTC", help="guest timezone VMX setting")
@click.option("--drag-and-drop/--no-drag-and-drop", is_flag=True, default=False, help="drag and drop")
@click.option("--clipboard/-no-clipboard", is_flag=True, default=False, help="copy and paste")
@click.option("--disk-growable", "disk_type", flag_value=0, help="growable single file virtual disk")
@click.option(
    "--disk-multifile-growable",
    "disk_type",
    flag_value=1,
    default=True,
    help="growable virtual disk split into multiple files",
)
@click.option("--disk-preallocated", "disk_type", flag_value=2, help="preallocated single file virtual disk")
@click.option(
    "--disk-multifle-preallocated",
    "disk_type",
    flag_value=3,
    help="preallocated virtual disk split into multiple files",
)
@click.option("--disk-esx", "disk_type", flag_value=4, help="preallocated ESX-type virtual disk")
@click.option("--disk-streaming", "disk_type", flag_value=5, help="compressed disk optimized for streaming")
@click.option("--disk-thin", "disk_type", flag_value=6, help="thin provisioned virtual disk")
@click.option("--serial", help="serial port named pipe")
@click.argument("name")
@click.pass_obj
def create(
    ctx,
    cpu,
    ram,
    disk,
    efi,
    mac,
    iso,
    no_iso,
    connect,
    start,
    gui,
    fullscreen,
    time_sync,
    drag_and_drop,
    clipboard,
    name,
    disk_type,
    serial,
    guest_timezone,
):
    """create VM"""
    iso = iso or ctx.iso
    if mac:
        mac = str(mac)
    if no_iso:
        iso = None
    result = ctx.create(
        name,
        cpu_count=cpu,
        ram_size=ram,
        disk_size=disk,
        disk_type=disk_type,
        iso_path=iso,
        iso_connected=connect,
        mac=mac,
        efi=efi,
        time_sync=time_sync,
        drag_and_drop=drag_and_drop,
        clipboard=clipboard,
        serial=serial,
        guest_timezone=guest_timezone,
    )
    if start:
        _start(ctx, gui, fullscreen, name)
        result.update(ctx.wait_power_state(name, "start", "poweredOn", click.echo))
    output(ctx, result)


@cli.command
@click.option("-f", "--force", is_flag=True, help="bypass confirmation")
@click.argument("name")
@click.pass_obj
def destroy(ctx, name, force):
    """destroy VM"""
    if not force:
        click.confirm(f"Confirm IRRECOVERABLE DESTRUCTION of VMWare Workstation '{name}' on '{ctx.host}'", abort=True)
    result = ctx.destroy(name, force)
    output(ctx, result)


@cli.command
@click.option("-e", "--enable", is_flag=True)
@click.option("-d", "--disable", is_flag=True)
@click.option("-p", "--port", type=int, help="VNC port (default: 5900 + <vm_ip> & 0xFF)")
@click.option("-a", "--address", type=IP_ADDRESS, default="127.0.0.1", show_default=True, help="listen IP address")
@click.argument("name")
@click.pass_obj
def vnc(ctx, name, enable, disable, port, address):
    """VM VNC settings"""
    result = ctx.vnc(name, enable, disable, port, address)
    output(ctx, result)


@cli.command
@click.option("-c/-C", "--copy/--no-copy", is_flag=True, default=None, help="clipboard cut")
@click.option("-p/-P", "--paste/--no-paste", is_flag=True, default=None, help="clipboard paste")
@click.option("-d/-D", "--dnd/--no-dnd", is_flag=True, default=None, help="drag and drop")
@click.option("-h/-H", "--hgfs/--no-hgfs", is_flag=True, default=None, help="shared folders")
@click.argument("name")
@click.pass_obj
def isolation(ctx, name, copy, paste, dnd, hgfs):
    """VM Isolation settings"""
    result = ctx.clipboard(name, copy, paste, dnd, hgfs)
    output(ctx, result)


@cli.command
@click.argument("name")
@click.pass_obj
def boot(ctx, name):
    """show boot parameters"""
    result = ctx.boot(name)
    output(ctx, result)


@cli.command
@click.option("-u", "--update-host-keys", is_flag=True, help="update ssh known_host keys")
@click.argument("name")
@click.pass_obj
def ssh(ctx, name, update_host_keys):
    """ssh config"""
    ssh = SSHConfig(ctx, name)
    if update_host_keys:
        return ssh.update()
    else:
        ssh.exec()


if __name__ == "__main__":
    sys.exit(cli())  # pragma: no cover



# local ssh config update

import os
from pathlib import Path
from subprocess import run

from .vmware_workstation import VMFailure


class SSHConfig:
    def __init__(self, ctx, name):
        self.name = name
        status = ctx.get_status(name)
        self.ip = status["ip"]
        self.mac = status["mac"]
        if not self.ip:
            raise VMFailure(f"{name} has no ip")
        if not self.mac:
            raise VMFailure(f"{name} has no mac address")

    def run(self, cmd, **kwargs):
        fail_ok = kwargs.pop("fail_ok", False)
        kwargs.setdefault("capture_output", True)
        kwargs.setdefault("text", True)
        kwargs.setdefault("check", True)
        proc = run(cmd, **kwargs)
        if proc.returncode != 0 and fail_ok is False:
            raise VMFailure(proc.stderr)
        return proc

    def update(self):
        known_hosts = Path.home() / ".ssh/known_hosts"
        self.run(["ssh-keygen", "-f", str(known_hosts), "-R", self.name], fail_ok=True)
        self.run(["ssh-keygen", "-f", str(known_hosts), "-R", self.ip], fail_ok=True)
        self.run(["sh", "-c", f"ssh-keyscan {self.name} >> {str(known_hosts)}"])
        self.run(["ssh", "-o", "UpdateHostKeys=yes", "-o", "StrictHostKeyChecking=no", self.name, "uptime"])

    def exec(self):
        os.execvp("ssh", ["ssh", self.name])



import json
from pathlib import Path

import pytest

import rstms_vmwctl.vmware_workstation


def _client():
    config_file = Path("tests/data/config.conf")
    config_text = config_file.read_text()
    config = json.loads(config_text)
    return rstms_vmwctl.vmware_workstation.Client(
        config["host"],
        config["url"],
        config["username"],
        config["password"],
        config["path"],
        config["arp_command"],
        config["iso"],
        config["mount_path"],
        config["ca_file"],
    )


@pytest.fixture
def client():
    api = _client()
    assert api
    return api


@pytest.fixture(scope="session")
def test_vm():
    client = _client()
    name = "test_vm"
    client.destroy(name, force=True)
    client.create(name)
    client.set_power(name, "on")
    client.wait_power_state(name, "on", "poweredOn")
    yield name
    client.set_power(name, "off")
    client.wait_power_state(name, "off", "poweredOff")
    client.destroy(name, force=True)


@pytest.fixture(scope="session")
def test_vm_api():
    client = _client()
    name = "test_vm_api"
    client.destroy(name, force=True)
    client.create(name)
    client.set_power(name, "on")
    client.wait_power_state(name, "on", "poweredOn")
    yield name
    client.set_power(name, "off")
    client.wait_power_state(name, "off", "poweredOff")
    client.destroy(name, force=True)


@pytest.fixture(scope="session")
def test_vm_gui():
    client = _client()
    name = "test_vm_gui"
    client.destroy(name, force=True)
    client.create(name)
    client.set_power(name, "start", gui=True)
    client.wait_power_state(name, "start", "poweredOn")
    yield name
    client.set_power(name, "off")
    client.wait_power_state(name, "off", "poweredOff")
    client.destroy(name, force=True)



# vmx template

from pathlib import Path


def _flag(value):
    return "TRUE" if value else "FALSE"


class VMX:

    template = """
.encoding = "UTF-8"
config.version = "8"
virtualHW.version = "19"
displayName = "{{NAME}}"
numvcpus = "{{CPU}}"
memsize = "{{RAM}}"
guestOS = "{{GUEST}}"
pciBridge0.present = "TRUE"
pciBridge4.present = "TRUE"
pciBridge4.virtualDev = "pcieRootPort"
pciBridge4.functions = "8"
pciBridge5.present = "TRUE"
pciBridge5.virtualDev = "pcieRootPort"
pciBridge5.functions = "8"
pciBridge6.present = "TRUE"
pciBridge6.virtualDev = "pcieRootPort"
pciBridge6.functions = "8"
pciBridge7.present = "TRUE"
pciBridge7.virtualDev = "pcieRootPort"
pciBridge7.functions = "8"
nvme0.present = "TRUE"
nvme0:0.fileName = "{{DISK}}"
nvme0:0.present = "TRUE"
floppy0.present = "FALSE"
ide1:0.present = "FALSE"
ethernet0.present = "FALSE"
vmx.scoreboard.enabled = "FALSE"
tools.syncTime = "{{TIME_SYNC}}"
time.synchronize.continue = "{{TIME_SYNC}}"
time.synchronize.restore = "{{TIME_SYNC}}"
time.synchronize.resume.disk = "{{TIME_SYNC}}"
time.synchronize.shrink = "{{TIME_SYNC}}"
time.synchronize.tools.startup = "{{TIME_SYNC}}"
guestTimeZone = "{{GUEST_TIMEZONE}}"
isolation.tools.dnd.disable = "{{DRAG_AND_DROP_DISABLE}}"
isolation.tools.copy.disable = "{{CLIPBOARD_COPY_DISABLE}}"
isolation.tools.paste.disable = "{{CLIPBOARD_PASTE_DISABLE}}"
"""

    def __init__(self, vmx_path):
        self.path = Path(vmx_path)

    def generate(
        self,
        *,
        cpu_count,
        ram_mb,
        guest_os="other-64",
        efi=False,
        time_sync=False,
        drag_and_drop=False,
        clipboard=False,
        guest_timezone="UTC",
    ):
        self.text = self.template
        self._apply("NAME", self.path.stem)
        self._apply("CPU", str(cpu_count))
        self._apply("RAM", str(ram_mb))
        self._apply("DISK", self.path.with_suffix(".vmdk").name)
        self._apply("GUEST", guest_os)
        self._apply("TIME_SYNC", _flag(time_sync))
        self._apply("GUEST_TIMEZONE", guest_timezone)
        self._apply("DRAG_AND_DROP_DISABLE", _flag(not drag_and_drop))
        self._apply("CLIPBOARD_COPY_DISABLE", _flag(not clipboard))
        self._apply("CLIPBOARD_PASTE_DISABLE", _flag(not clipboard))
        self.set_cdrom(None, False)
        self.set_ethernet(False)
        self.set_serial(False)
        if efi:
            self.text += '\nfirmware = "efi"'

    def _apply(self, macro, value):
        self.text = self.text.replace("{{" + macro + "}}", value)

    def set_cdrom(self, iso_pathname=None, connected=False):
        self.iso_pathname = iso_pathname
        self.iso_connected = connected
        self._update("ide1", self._cdrom)

    def _cdrom(self):
        lines = []
        if self.iso_pathname is not None:
            lines.append('ide1:0.present = "TRUE"')
            lines.append('ide1:0.deviceType = "cdrom-image"')
            lines.append(f'ide1:0.fileName = "{str(self.iso_pathname)}"')
            if self.iso_connected:
                lines.append('ide1:0.startConnected = "TRUE"')
            else:
                lines.append('ide1:0.startConnected = "FALSE"')
        else:
            lines.append('ide1:0.present = "FALSE"')
        return lines

    def set_ethernet(self, present=True, mac=None):
        self.ethernet = present
        self.mac = mac
        self._update("ethernet", self._ethernet)

    def _ethernet(self):
        lines = []
        if self.ethernet:
            lines.append('ethernet0.present = "TRUE"')
            lines.append('ethernet0.virtualDev = "e1000"')
            if self.mac:
                lines.append(f'ethernet0.address = "{self.mac}"')
                lines.append('ethernet0.addressType = "static"')
            else:
                lines.append('ethernet0.addressType = "generated"')
        else:
            lines.append('ethernet0.present = "FALSE"')
        return lines

    def set_serial(self, present=True, pipe=None):
        self.serial = present
        self.pipe = pipe
        self._update("serial", self._serial)

    def _serial(self):
        lines = []
        if self.serial:
            lines.append('serial0.present = "TRUE"')
            lines.append('serial0.fileType = "pipe"')
            lines.append(f'serial0.fileName = "\\\\.\\pipe\\{self.pipe}"')
            lines.append('serial0.pipe.endPoint = "client"')
            lines.append('serial0.tryNoRxLoss = "TRUE"')
        else:
            lines.append('serial0.present = "FALSE"')
        return lines

    def _update(self, prefix, add_lines):
        lines = []
        updated = False
        for line in self.text.strip().split("\n"):
            line = line.strip()
            if not line:
                continue
            if line.startswith(prefix):
                continue
            if line.startswith("uuid"):
                if not updated:
                    lines.extend(add_lines())
                    updated = True
            lines.append(line)
        if not updated:
            lines.extend(add_lines())
        self.text = "\n".join(lines)

    def read(self):
        self.text = self.path.read_text()

    def write(self):
        self.path.write_text(self.text)



import re
import shlex
import shutil
import socket
import subprocess
import time
from pathlib import Path
from urllib.parse import quote

import requests

from .vmx import VMX

MAC_PATTERN = r"([0-9a-f]{2}:){5}[0-9a-f]{2}$"


class VMFailure(Exception):
    pass


class VM:
    def __init__(self, name=None, vmx=None, id=None, mac=None, ip=None, power=None, dict=None):
        self.name = name
        self.vmx = Path(vmx) if vmx else None
        self.id = id
        self.mac = mac
        self.ip = ip
        self.power = power
        if dict is not None:
            self.from_dict(dict)

    def to_dict(self):
        return dict(name=self.name, vmx=str(self.vmx), id=self.id, mac=self.mac, ip=self.ip, power=self.power)

    def from_dict(self, data):
        self.name = data["name"]
        vmx = data["vmx"]
        self.vmx = Path(vmx) if vmx else None
        self.id = data.get("id", None)
        self.mac = data.get("mac", None)
        self.ip = data.get("ip", None)
        self.power = data.get("power", None)


class Client:

    cpu_count = 1
    disk_size = 8192
    ram_size = 1024
    timeout = 60
    wait = True
    protected_names = ["phobos", "cryptovault"]

    def __init__(
        self, host, url, username, password, path_list, arp_command, iso, mount_path, ca_file, client_cert, client_key
    ):
        self.host = host
        self.url = url
        self.username = username
        self.password = password
        self.arp_command = arp_command
        self.iso = iso
        self.vm = {}
        self.arp_table = None
        self.vmrun = False
        self.verbose = False
        self.raw = False
        self.vmware_path_list = [Path(p) for p in path_list.split(",")]
        self.vmware_path = self.vmware_path_list[0]
        self.mount_path = mount_path
        self.ca_file = ca_file
        self.client_cert = client_cert
        self.client_key = client_key

    def wait_power_state(self, name, operation, power_state, echo=None):
        if echo is None:
            self.verbose = False
        if self.wait:
            self.vmrun = False
            if self.verbose:
                echo(operation, err=True, nl=False)
            timeout = time.time() + self.timeout
            while True:
                if self.verbose:
                    echo(".", err=True, nl=False)
                time.sleep(1)
                if name in self.get_vms():
                    if self.power_state(name) == power_state:
                        break
                if time.time() > timeout:
                    raise VMFailure(f"timed out after {self.timeout} seconds awaiting '{power_state}'")
            if self.verbose:
                echo(err=True)
        return self.get_status(name)

    def _host_path(self, path, enforce_file=None, enforce_absent=None):
        path = Path(path)
        if enforce_file:
            if not path.is_file():
                raise VMFailure(f"file not found: {str(path)}")
        if enforce_absent:
            if path.exists():
                raise VMFailure(f"path exists: {str(path)}")
        if str(path).startswith(self.mount_path):
            path = str(path)[len(self.mount_path) :]  # noqa: E203
            drive, _, path = path.partition("/")
            path = drive.upper() + ":/" + path
        path = str(path).replace("/", "\\")
        return Path(path)

    def _local_path(self, path):
        if not path:
            return ""
        path = str(path)
        path = path.replace("\\", "/")
        if path[1] == ":":
            path = "/mnt/beaker/" + path[0].lower() + path[2:]
        return Path(path)

    def _vmrun(self, cmd, strip=True, split=None):
        if isinstance(cmd, str):
            cmd = shlex.split(cmd)
        proc = self._run(
            ["vmrun", "-T", "ws"] + cmd,
            return_proc=True,
        )
        if proc.returncode == 0:
            ret = proc.stdout
            if strip:
                ret = ret.strip()
            if split:
                ret = ret.split(split)
            return ret
        else:
            raise VMFailure("\n".join([proc.stderr.strip() + proc.stdout.strip()]))

    def _run(self, cmd, **kwargs):
        if isinstance(cmd, str):
            cmd = shlex.split(cmd)
        strip = kwargs.pop("strip", True)
        split = kwargs.pop("split", False)
        return_proc = kwargs.pop("return_proc", False)
        kwargs.setdefault("text", True)
        kwargs.setdefault("check", not return_proc)
        kwargs.setdefault("capture_output", True)
        proc = subprocess.run(["ssh", self.host] + cmd, **kwargs)
        if return_proc:
            return proc
        ret = proc.stdout
        if strip:
            ret = ret.strip()
        if split:
            ret = ret.split(split)
        return ret

    def _request(self, func, path, **kwargs):
        raise_on_failure = kwargs.pop("raise_on_failure", True)
        parse_json = kwargs.pop("parse_json", True)
        kwargs["auth"] = (self.username, self.password)
        kwargs["headers"] = {
            "Content-Type": "application/vnd.vmware.vmw.rest-v1+json",
            "Accept": "application/vnd.vmware.vmw.rest-v1+json",
        }
        result = func(f"{self.url}/api/{path}", **kwargs)
        if raise_on_failure:
            result.raise_for_status()
        if parse_json:
            return result.json()
        return result

    def get(self, path, **kwargs):
        return self._request(requests.get, path, **kwargs)

    def delete(self, path, **kwargs):
        return self._request(requests.delete, path, **kwargs)

    def put(self, path, **kwargs):
        return self._request(requests.put, path, **kwargs)

    def post(self, path, **kwargs):
        return self._request(requests.put, path, **kwargs)

    def get_vms_api(self):
        self.vm = {}
        vms = self.get("vms")
        for vm in vms:
            path = self._local_path(vm["path"])
            name = path.stem
            self.vm[name] = VM(name=name, vmx=path, id=vm["id"])
        return self.vm

    def get_vms_vmrun(self):
        ret = {}
        lines = self._vmrun("list", split="\n")
        for line in lines:
            if line.startswith("Total"):
                continue
            path = Path(self._local_path(line))
            name = path.stem
            vm = VM(name=name, vmx=path)
            cvm = self.vm.get(name, None)
            if cvm:
                if cvm.vmx != vm.vmx:
                    raise VMFailure(f"vmx path mismatch: {name}")
                vm.id = cvm.id
            ret[name] = vm
        return ret

    def get_vms(self):
        if self.vmrun:
            return self.get_vms_vmrun()
        else:
            return self.get_vms_api()

    def get_vm(self, name, must_exist=True):
        vms = self.get_vms()
        try:
            return vms[name]
        except KeyError:
            raise VMFailure(f"not found: {name}")

    def get_vid(self, name):
        vm = self.get_vm(name)
        return vm.id

    def get_status(self, name):
        vm = self.get_vm(name).to_dict()
        vm.update(self.get_power(None, vm["id"]))
        vm.update(self.get_mac(None, vm["id"]))
        if vm["mac"]:
            vm.update(self.get_ip(None, mac=vm["mac"]))
        else:
            vm["ip"] = ""
        return vm

    def get_detail(self, name, vid=None):
        vid = vid or self.get_vid(name)
        config = self.get(f"vms/{vid}/restrictions")
        return config

    def get_vmx_files(self):
        return [str(f) for f in self._vmx_files()]

    def _vmx_files(self):
        ret = []
        for vmware_dir in self.vmware_path_list:
            names = [item.name for item in vmware_dir.iterdir() if item.is_dir()]
            files = [vmware_dir / name / f"{name}.vmx" for name in names]
            ret.extend([f for f in files if f.is_file()])
        return ret

    def _find_vmx_file(self, name):
        for file in self._vmx_files():
            if file.stem == name:
                return file
        raise VMFailure(f"vmx not found: {name}")

    def _gui_start(self, vmx, fullscreen):
        cmd = ["cmd", "vmware", "-n", "-q"]
        if fullscreen:

            cmd.append("-X")
        else:
            cmd.append("-x")
        cmd.append(str(vmx))
        proc = subprocess.Popen(cmd, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
        return proc.pid

    def _power_result(self, operation):
        if operation == "start":
            return "vm_started"
        elif operation == "stop":
            return "vm_stopped"
        elif operation == "reboot":
            return "vm_rebooted"
        elif operation == "on":
            return "vm_powered_up"
        elif operation == "off":
            return "vm_powered_down"
        elif operation == "shutdown":
            return "vm_shutdown"
        return operation

    def set_power(self, name, operation, hard=False, gui=False, fullscreen=False):
        ret = dict(name=name)
        if operation in ["start", "stop", "reboot"]:
            vmx = self._host_path(self._find_vmx_file(name), enforce_file=True)
            if operation == "start" and (gui or fullscreen):
                ret["mode"] = "vmware"
                self.set_parameter(name, "gui.enableStretchGuest", "TRUE" if fullscreen else "FALSE")
                ret["pid"] = self._gui_start(vmx, fullscreen)
            else:
                cmd = [operation, f"'{vmx}'"]
                if operation in ["stop", "reset"]:
                    if hard:
                        cmd.append("hard")
                    else:
                        cmd.append("soft")
                ret["result"] = self._vmrun(cmd)
                ret["mode"] = "vmrun"
        elif operation in ["on", "off", "shutdown"]:
            self.vmrun = False
            vid = self.get_vid(name)
            ret["id"] = vid
            ret["mode"] = "api"
            state = self.put(f"vms/{vid}/power", data=operation.encode())
            ret["power"] = state["power_state"]
        else:
            raise VMFailure(f"unexpected power operation: {operation}")

        ret["result"] = self._power_result(operation)
        return ret

    def get_power(self, name, vid=None):
        vid = vid or self.get_vid(name)
        ret = self.get(f"vms/{vid}/power")
        return dict(power=ret["power_state"])

    def get_mac(self, name, vid=None, fail_ok=True):
        if self.vmrun:
            vm = self.get_vm(name)
            path = self._host_path(vm.vmx, enforce_file=True)
            return dict(mac=self._vmrun(["readVariable", f"'{path}'", "runtimeConfig", "ethernet0.generatedAddress"]))
        else:
            vid = vid or self.get_vid(name)
            result = self.get(f"vms/{vid}/nic")
            for nic in result.get("nics", []):
                return dict(mac=nic.get("macAddress"))

        if fail_ok:
            return dict(mac="")

        raise VMFailure("no mac address")

    def get_ip(self, name, vid=None, mac=None, fail_ok=True):

        if self.vmrun:
            vm = self.get_vm(name)
            path = self._host_path(vm.vmx, enforce_file=True)
            return dict(ip=self._vmrun(["getGuestIpAddress", f"'{path}'"]))

        mac = mac or self.get_mac(name, vid=vid)["mac"]
        if mac:
            if self.arp_table is None:
                self.arp_table = (
                    subprocess.run(shlex.split(self.arp_command), text=True, capture_output=True, check=True)
                    .stdout.strip()
                    .split("\n")
                )
            for arp in self.arp_table:
                m = re.match(r"(\S+)\s+" + mac + r".*", arp, re.I)
                if m:
                    return dict(ip=m.groups()[0])

        if fail_ok:
            return dict(ip="")

        raise VMFailure("no IP address")

    def _vm_exists(self, name):
        self.get_vms()
        return name in self.vm

    def _vmx_path(self, name, enforce_exists=None, enforce_absent=None, create=False):
        path = self.vmware_path / name
        if path.exists():
            if enforce_absent:
                raise VMFailure(f"vmx path '{str(path)}' exists")
            if not path.is_dir():
                raise VMFailure(f"vmx path '{str(path)}' is not a directory")
        else:
            if enforce_exists:
                raise VMFailure(f"vmx path '{str(path)}' does not exist")
            if create:
                path.mkdir()
        return path

    def _host_env(self):
        """return host env as dict"""
        vars = self._run(["set"], split="\n")
        ret = {}
        for var in vars:
            name, _, value = var.partition("=")
            ret[name] = value
        return ret

    def _create_disk_image(self, vmx_path, name, disk_size, disk_type):
        remote_os = self._host_env().get("OS", "unknown")
        if "windows" in remote_os.lower():
            ssh_path = str(vmx_path).replace(self.mount_path, "")
            drive, _, path = ssh_path.partition("/")
            cmd = f"{drive}:&"
            for element in path.split("/"):
                cmd += f"cd {element}&"
            cmd += f"vmware-vdiskmanager -c -s {disk_size}MB -a nvme -t {disk_type} {name}.vmdk"
        else:
            raise RuntimeError(f"Unsupported host OS: {remote_os}")

        proc = self._run(cmd, return_proc=True)
        if proc.returncode != 0:
            raise VMFailure("failure creating disk image: {proc.stderr}")
        return f"{name}.vmx"

    def create(
        self,
        name,
        *,
        cpu_count=None,
        ram_size=None,
        disk_size=None,
        disk_type=1,
        iso_path=None,
        iso_connected=True,
        mac=None,
        efi=False,
        time_sync=True,
        clipboard=False,
        drag_and_drop=False,
        serial=None,
        guest_timezone="UTC",
    ):

        cpu_count = cpu_count or self.cpu_count
        ram_size = ram_size or self.ram_size
        disk_size = disk_size or self.disk_size
        iso_path = iso_path or self.iso

        if mac:
            mac = mac.strip('"')
            mac = mac.strip("'")
            if not re.match(MAC_PATTERN, mac):
                raise VMFailure(f"Unrecognized mac format: '{mac}'")

        if self._vm_exists(name):
            raise VMFailure(f"VM '{name}' exists")

        vmx_path = self._vmx_path(name, enforce_absent=True, create=True)
        self._create_disk_image(vmx_path, name, disk_size, disk_type)
        vmx_file = vmx_path / f"{name}.vmx"

        vmx = VMX(vmx_file)
        vmx.generate(
            cpu_count=cpu_count,
            ram_mb=ram_size,
            efi=efi,
            time_sync=time_sync,
            drag_and_drop=drag_and_drop,
            clipboard=clipboard,
            guest_timezone=guest_timezone,
        )
        # configure ethernet if dynamic MAC
        if not mac:
            vmx.set_ethernet(True, mac)
        vmx.write()
        vm = self.get_vm(name)
        if str(vm.vmx) != str(vmx_file):
            raise VMFailure("vmx path mismatch")

        # start/stop VM so vmware can set uuid.bios, uuid.location
        on_status = self.set_power(name, "on")
        if on_status["power"] != "poweredOn":
            raise VMFailure(f"unexpected power state: {on_status['power']}")
        off_status = self.set_power(name, "off")
        if off_status["power"] != "poweredOff":
            raise VMFailure(f"unexpected power state: {off_status['power']}")
        vmx.read()

        # configure cdrom
        if iso_path:
            iso_path = self.check_iso_path(iso_path)
            host_iso_path = self._host_path(iso_path, enforce_file=True)
        else:
            host_iso_path = None
            iso_connected = False
        vmx.set_cdrom(host_iso_path, iso_connected)

        # configure ethernet
        if mac:
            vmx.set_ethernet(True, mac)

        if serial is not None:
            vmx.set_serial(True, serial)

        vmx.write()
        ret = self.get_status(name)
        ret.update(dict(result="created"))
        return ret

    def register(self, name, vmx_path):
        host_vmx_path = self._host_path(vmx_path, enforce_file=True)
        ret = self.post("vms/registration", json=dict(name=name, path=host_vmx_path))
        return ret

    def destroy(self, name, force):  # noqa: C901
        ret = dict(name=name)
        self.vmrun = False
        if self._vm_exists(name):
            ret["result"] = "vm_destroyed"
            vid = self.get_vid(name)
            if force:
                if self.power_state(None, vid) != "poweredOff":
                    self.set_power(name, "off")
            else:
                self.check_power_off(None, vid)
            ret["id"] = vid
            response = self.delete(f"/vms/{vid}", parse_json=False)
            if not response.ok:
                raise VMFailure(f"{response} {response.reason}")

        path = self._vmx_path(name)
        if path.exists():
            if path.is_dir():
                ret["result"] = "vmx_path_deleted"
                for name in self.protected_names:
                    assert name not in str(path)
                ret["path"] = str(path)
                if force:
                    try:
                        shutil.rmtree(path)
                    except Exception as e:
                        if type(e) is FileNotFoundError:
                            raise
                else:
                    raise VMFailure("vmx path not deleted without --force")
            else:
                raise VMFailure(f"vmx path is not a directory: {str(path)}")
        if "result" not in ret:
            ret["result"] = "vm_nonexistent"
        self.vm.pop(name, None)
        return ret

    def power_state(self, name, vid=None):
        return self.get_power(name, vid)["power"]

    def check_power_off(self, name, vid=None):
        vid = vid or self.get_vid(name)
        if self.power_state(None, vid) != "poweredOff":
            raise VMFailure("VM must be powered off")
        return vid

    def set_parameter(self, name, key, value):
        if self.vmrun:
            vm = self.get_vm(name)
            path = self._host_path(vm.vmx, enforce_file=True)
            return self._vmrun(["writeVariable", f"'{path}'", key, value])
        else:
            vid = self.get_vid(name)
            return self.put(f"vms/{vid}/params", json=dict(name=key, value=value), parse_json=False)

    def _download_iso(self, url):
        m = re.match(r".*/([^/]*\.iso)$", url)
        filename = m.groups()[0]
        pathname = self.vmware_path / "iso" / filename
        kwargs = {}
        if self.ca_file:
            kwargs["verify"] = self.ca_file
        if self.client_cert and self.client_key:
            kwargs["cert"] = (self.client_cert, self.client_key)
        response = requests.get(url, **kwargs)
        response.raise_for_status()
        with open(pathname, "wb") as file:
            for chunk in response.iter_content(chunk_size=8192):
                file.write(chunk)
        return Path(pathname)

    def check_iso_path(self, iso_path):
        """check if iso_path is a file or download if URL, return path"""
        p = re.compile(r"^https*://.*", re.I)
        if p.match(iso_path):
            iso_path = self._download_iso(iso_path)
        else:
            iso_path = Path(iso_path)
            if not iso_path.is_file():
                raise VMFailure(f"iso_path is not a valid file or URL: '{str(iso_path)}'")
        return iso_path

    def cdrom(self, name, iso_path=None, connected=None):
        ret = dict(result="failure", detail="unknown")
        vid = self.get_vid(name)
        if iso_path:
            iso_path = self.check_iso_path(iso_path)
            host_iso_path = self._host_path(iso_path, enforce_file=True)
            self.put(f"vms/{vid}/params", json=dict(name="ide1:0.present", value="TRUE"), parse_json=False)
            self.put(f"vms/{vid}/params", json=dict(name="ide1:0.fileName", value=str(host_iso_path)), parse_json=False)
            if connected is True:
                self.put(f"vms/{vid}/params", json=dict(name="ide1:0.startConnected", value="TRUE"), parse_json=False)
            elif connected is False:
                self.put(f"vms/{vid}/params", json=dict(name="ide1:0.startConnected", value="FALSE"), parse_json=False)
        else:
            self.put(f"vms/{vid}/params", json=dict(name="ide1:0.present", value="FALSE"), parse_json=False)
            self.put(f"vms/{vid}/params", json=dict(name="ide1:0.fileName", value=""), parse_json=False)
            self.put(f"vms/{vid}/params", json=dict(name="ide1:0.startConnected", value="FALSE"), parse_json=False)

        ret = {}
        ret["name"] = name
        ret["id"] = vid
        params = self.get(f"vms/{vid}/params/{quote("ide1:0.fileName")}")
        iso_path = params["value"]
        ret["iso"] = str(self._local_path(iso_path))
        params = self.get(f"vms/{vid}/params/{quote("ide1:0.present")}")
        ret["present"] = params["value"]
        params = self.get(f"vms/{vid}/params/{quote("ide1:0.startConnected")}")
        ret["start_connected"] = params["value"]

        return ret

    def get_vmx(self, name):
        vm = self.get_vm(name)
        return vm.vmx.read_text()

    def _vnc_port(self, vid):
        vmip = self.get_ip(None, vid=vid)
        ip = vmip.get("ip", "0.0.0.0")
        host = int(ip.split(".")[-1])
        port = 5900 + (host & 0xFF)
        return port

    def vnc(self, name, enable=False, disable=False, port=None, address=None):
        ret = {}
        vid = self.get_vid(name)
        ret["id"] = vid
        if enable:
            self.put(f"vms/{vid}/params", json=dict(name="RemoteDisplay.vnc.enabled", value="TRUE"), parse_json=False)
            port = port or self._vnc_port(vid)
            address = address or "127.0.0.1"
            if str(address) in ["0.0.0.0", "egress", "public"]:
                address = socket.gethostbyname(self.host)
        if disable:
            self.put(f"vms/{vid}/params", json=dict(name="RemoteDisplay.vnc.enabled", value="FALSE"), parse_json=False)
        if port:
            self.put(f"vms/{vid}/params", json=dict(name="RemoteDisplay.vnc.port", value=str(port)), parse_json=False)
        if address:
            self.put(f"vms/{vid}/params", json=dict(name="RemoteDisplay.vnc.ip", value=str(address)), parse_json=False)

        for param in ["enabled", "port", "ip"]:
            result = self.get(f"vms/{vid}/params/{'RemoteDisplay.vnc.' + param}")
            ret[param] = result["value"]

        return ret

    def serial(self, name, enable=False, pipe=None):
        ret = {}
        vid = self.get_vid(name)
        ret["id"] = vid

        if enable:
            self.put(f"vms/{vid}/params", json=dict(name="serial0.present", value="TRUE"), parse_json=False)
            self.put(f"vms/{vid}/params", json=dict(name="serial0.fileType", value="pipe"), parse_json=False)
            self.put(
                f"vms/{vid}/params", json=dict(name="serial0.fileName", value=f"\\\\.\\pipe\\{pipe}"), parse_json=False
            )
            self.put(f"vms/{vid}/params", json=dict(name="serial0.pipe.endPoint", value="client"), parse_json=False)
            self.put(f"vms/{vid}/params", json=dict(name="serial0.tryNoRxLoss", value="TRUE"), parse_json=False)
        else:
            self.put(f"vms/{vid}/params", json=dict(name="serial0.present", value="FALSE"), parse_json=False)

        for param in ["fileType", "fileName", "pipe.endPoint", "tryNoRxLoss", "present"]:
            result = self.get(f"vms/{vid}/params/{'serial0.' + param}")
            ret[param] = result["value"]

        return ret

    def clipboard(self, name, copy=None, paste=None, dnd=None, hgfs=None):
        ret = {}
        vid = self.get_vid(name)
        ret["id"] = vid
        params = {
            "isolation.tools.copy.disable": copy,
            "isolation.tools.paste.disable": paste,
            "isolation.tools.dnd.disable": dnd,
            "isolation.tools.hgfs.disable": hgfs,
        }
        for key, value in params.items():
            if value is not None:
                self.put(f"vms/{vid}/params", json=dict(name=key, value="FALSE" if value else "TRUE"), parse_json=False)

        detail = self.get_detail(name)
        isolation = detail["guestIsolation"]

        for key in params.keys():
            ret["copy"] = "FALSE" if isolation["copyDisabled"] else "TRUE"
            ret["paste"] = "FALSE" if isolation["pasteDisabled"] else "TRUE"
            ret["dnd"] = "FALSE" if isolation["dndDisabled"] else "TRUE"
            ret["hgfs"] = "FALSE" if isolation["hgfsDisabled"] else "TRUE"

        return ret

    def boot(self, name):
        ret = {}
        vid = self.get_vid(name)
        ret["id"] = vid

        for param in ["bootOrder", "hddOrder", "bootDeviceClasses"]:
            result = self.get(f"vms/{vid}/params/bios.{param}")
            ret[param] = result["value"]

        return ret



import pytest

from rstms_vmwctl.vmware_workstation import VM


def _check_vms(vms):
    assert isinstance(vms, dict)
    assert vms
    for k, v in vms.items():
        assert isinstance(k, str)
        assert isinstance(v, VM)
    return True


def test_vm_api_get_vms(client, test_vm_api):
    assert client.vmrun is False
    vms = client.get_vms()
    assert _check_vms(vms)
    assert test_vm_api in vms


def test_vm_vmrun_get_vms(client, test_vm_gui):
    client.vmrun = True
    vms = client.get_vms()
    assert _check_vms(vms)
    assert test_vm_gui in vms


def test_vm_api_get_mac(client, test_vm_api):
    assert client.vmrun is False
    mac = client.get_mac(test_vm_api)
    assert isinstance(mac, dict)
    assert "mac" in mac
    assert mac["mac"]


def test_vm_vmrun_get_mac(client, test_vm_gui):
    client.vmrun = True
    mac = client.get_mac(test_vm_gui)
    assert isinstance(mac, dict)
    assert "mac" in mac
    assert mac["mac"]


def test_vm_api_get_ip(client, test_vm_api):
    assert client.vmrun is False
    ip = client.get_ip(test_vm_api)
    assert isinstance(ip, dict)
    assert "ip" in ip
    assert ip["ip"]


@pytest.mark.skip(reason="disabled")
def test_vm_vmrun_get_ip(client, test_vm_gui):
    client.vmrun = True
    ip = client.get_ip(test_vm_gui)
    assert isinstance(ip, dict)
    assert "ip" in ip
    assert ip["ip"]


def test_vm_api_get_vmx(client, test_vm_api):
    assert client.vmrun is False
    vmx = client.get_vmx(test_vm_api)
    assert isinstance(vmx, str)
    assert vmx


def test_vm_vmrun_get_vmx(client, test_vm_gui):
    client.vmrun = True
    vmx = client.get_vmx(test_vm_gui)
    assert isinstance(vmx, str)
    assert vmx



import pytest

from rstms_vmwctl.vmx import VMX


@pytest.fixture
def vmx(datadir):
    vmx_file = datadir / "vmxtest.vmx"
    return VMX(vmx_file)


def test_vmx_gen(vmx):
    vmx.generate(
        cpu_count=2,
        ram_mb=256,
        vmdk_pathname="vmdk_pathname",
    )
    assert vmx.text
    assert 'numvcpus = "2"' in vmx.text
    assert 'memsize = "256"' in vmx.text
    assert 'nvme0.present = "TRUE"' in vmx.text
    assert 'nvme0:0.fileName = "vmdk_pathname"' in vmx.text



import pytest

from rstms_vmwctl.sshconfig import SSHConfig


@pytest.fixture
def name():
    return "opensea"


@pytest.fixture
def ssh(client, name):
    return SSHConfig(client, name)


def test_ssh_update(ssh):
    ssh.update()



"""
  ehandler

  well-behaved exception handler for python cli commands

"""

import sys
from traceback import format_exception

from .vmware_workstation import VMFailure


def exception_handler(
    exception_type,
    exception,
    traceback,
    debug_hook=sys.excepthook,
):
    logger = ExceptionHandler.logger

    elist = format_exception(exception)
    traceback_msg = "".join(elist[:-1]).rstrip("\n")
    error_msg = elist[-1].rstrip("\n")

    if logger:
        logger.debug(traceback_msg)
        logger.error(error_msg)

    if ExceptionHandler.debug:
        debug_hook(
            exception_type,
            exception,
            traceback,
        )

    elif not logger:
        if exception_type == VMFailure:
            error_msg = f"vmctl: {' '.join(exception.args)}"
        print(error_msg, file=sys.stderr, end="\n", flush=True)

    sys.exit(-1)


class ExceptionHandler:
    installed = False
    debug = None
    logger = None

    def __init__(self, debug=False, logger=False):
        self.__class__.debug = debug
        self.__class__.logger = logger
        if not self.installed:
            sys.excepthook = exception_handler
            self.__class__.installed = True
